#version 330

layout(std140) uniform Matrices
{
  mat4 mMat;
  mat4 vMat;
  mat4 pMat;
};

layout(std140) uniform DirectionalLight
{
  vec4 color;
  vec4 direction;
  float intensity;
}
directionalLight;

layout(std140) uniform Material
{
  vec4 ambient;
  vec4 diffuse;
  vec4 specular;
  float shininess;
}
material;

in gsData
{
  vec3 fragPos;
  vec3 normal;
  vec3 baryCoord;
}
gsOut;

out vec4 fragColor;

vec4 calcDirLight(vec3 fragPos, vec3 normal);
float edgeFactor();

void main()
{
  //if (gsOut.baryCoord.x < 0.01 || gsOut.baryCoord.y < 0.01 ||
  //    gsOut.baryCoord.z < 0.01)
  //{
  //  fragColor = vec4(0, 0, 0, 1);
  //}
  //else
  //{
  //  fragColor = calcDirLight(gsOut.fragPos, gsOut.normal);
  //}
	float d = edgeFactor();
	if (d < 0.3)
	{
    fragColor = vec4(0, 0, 0, 1);
	}
	else
	{
    fragColor = calcDirLight(gsOut.fragPos, gsOut.normal);
	}
}

float edgeFactor()
{
	vec3 d = fwidth(gsOut.baryCoord);
	vec3 a3 = smoothstep(vec3(0.0), d*1.5, gsOut.baryCoord);
	return min(min(a3.x, a3.y), a3.z);
}

vec4 calcDirLight(vec3 fragPos, vec3 normal)
{
  // TODO matrix mult on cpu side to eye space
  vec3 lightDir = -normalize((vMat * directionalLight.direction).xyz);
  // diffuse shading
  float diff = max(dot(normal, lightDir), 0.0);
  // specular shading
  vec3 cameraDir = normalize(fragPos);
  vec3 reflectDir = reflect(lightDir, normal);
  float spec = pow(max(dot(cameraDir, reflectDir), 0.0), material.shininess);
  // combine results
  vec3 ambient = vec3(directionalLight.color) * vec3(material.ambient) * 0.5;
  vec3 diffuse = vec3(directionalLight.color) * diff * vec3(material.diffuse);
  vec3 specular = vec3(1, 1, 1) * spec * vec3(material.specular);
  return vec4(ambient + diffuse + specular, 1);

  // return vec4(diffuse, 1);

  // return (vec4(normal,1));
  // return vec4(1,0,0,1);
}
